import yaml


class OpenStackResource:
    def __init__(self, client):
        self.client = client

    def _exec(self, method, *args, yaml_output=True):
        """
        :param method: operation on resource i.e "list", "create"
        :param args: list of string positional args should go first i.e.:
            ["--flavor=m1.tiny"]
            ["vm-test-0", "--flavor=m1.tiny"]
        """
        command = ['openstack', self.resource, method, *args]

        if yaml_output:
            command += ['-f', 'yaml']
            return yaml.safe_load(
                self.client.exec(command))
        return self.client.exec(command)

    def list(self, args, yaml_output=True):
        return self._exec('list', *args, yaml_output=yaml_output)

    def create(self, args, yaml_output=True):
        return self._exec('create', *args, yaml_output=yaml_output)

    def delete(self, args, yaml_output=False):
        return self._exec('delete', *args, yaml_output=yaml_output)

    def show(self, args):
        return self._exec('show', *args)

    def set(self, args):
        return self._exec('set', *args, yaml_output=False)

    def add(self, args, yaml_output=False,):
        return self._exec('add', *args, yaml_output=yaml_output)

    @property
    def resource(self):
        return self._resource


class LoadBalancer(OpenStackResource):
    _resource = 'loadbalancer'

    def pool_list(self, args, yaml_output=True):
        return self._exec('pool list',
                          yaml_output=yaml_output,
                          *args)


class ZoneBlacklist(OpenStackResource):
    _resource = 'zone blacklist'


class Role(OpenStackResource):
    _resource = 'role'


class User(OpenStackResource):
    _resource = 'user'


class Image(OpenStackResource):
    _resource = 'image'


class Server(OpenStackResource):
    _resource = 'server'


class Flavor(OpenStackResource):
    _resource = 'flavor'


class Stack(OpenStackResource):
    _resource = 'stack'

    def resource_list(self, args, yaml_output=False,):
        return self._exec('resource list',
                          yaml_output=yaml_output,
                          *args)

    def event_list(self, args, yaml_output=False,):
        return self._exec('event list',
                          yaml_output=yaml_output,
                          *args)


class ResourceProvider(OpenStackResource):
    _resource = "resource provider"


class ResourceProviderUsage(OpenStackResource):
    _resource = "resource provider usage"


class Network(OpenStackResource):
    _resource = 'network'


class Host(OpenStackResource):
    _resource = 'host'


class Notification(OpenStackResource):
    _resource = 'notification'


class Hypervisor(OpenStackResource):
    _resource = 'hypervisor'


class ComputeService(OpenStackResource):
    _resource = 'compute service'


class Subnet(OpenStackResource):
    _resource = 'subnet'


class Segment(OpenStackResource):
    _resource = 'segment'

    def host_create(self, args, yaml_output=False,):
        return self._exec('host create',
                          yaml_output=yaml_output,
                          *args)

    def host_update(self, args, yaml_output=False, ):
        return self._exec('host update',
                          yaml_output=yaml_output,
                          *args)

    def host_list(self, args, yaml_output=True, ):
        return self._exec('host list',
                          yaml_output=yaml_output,
                          *args)


class Volume(OpenStackResource):
    _resource = 'volume'


class Secret(OpenStackResource):
    _resource = 'secret'
