import os
from si_tests.deployments.utils import (kubectl_utils, commons,
                                        file_utils, wait_utils)
from si_tests.deployments.utils.namespace import NAMESPACE


def deploy_refapp(openstack_client_manager):
    """KUBECONFIG is required"""

    stack_name = "refapp-test"
    source = file_utils.join(commons.REFAPP_DIR, "heat-templates")
    if not os.path.exists(source):
        commons.LOG.warning("repapp source does not exist %s", source)

    client_name = openstack_client_manager.client.name
    kubectl = kubectl_utils.Kubectl()
    destination = "{}/{}:{}".format(
        NAMESPACE.openstack, client_name, "/tmp/heat-templates")

    commons.LOG.info("Copy heat templates")
    kubectl.cp(source, destination)

    commons.LOG.info("Create heat stack")
    openstack_client_manager.stack.create(
        [stack_name, "-t", "/tmp/heat-templates/top.yaml"])

    commons.LOG.info("Wait for heat stack to create")
    wait = wait_utils.Waiter(openstack_client_manager.os_manager, 3600)

    def f():
        status = openstack_client_manager.stack.show(
            [stack_name])["stack_status"]
        if status == "CREATE_FAILED":
            resource_list = openstack_client_manager.stack.resource_list(
                ['-n', '10', stack_name])
            event_list = openstack_client_manager.stack.event_list(
                ['--nested-depth', '10', stack_name])
            commons.LOG.error("Resource info: %s\nHeat stack event list: %s",
                              resource_list, event_list)
            raise Exception("Failed to create stack")
        return status == "CREATE_COMPLETE"

    wait.wait(f)

    commons.LOG.info("Refapp successfully deployed")


def delete_refapp(openstack_client_manager):
    """KUBECONFIG is required"""

    stack_name = "refapp-test"

    commons.LOG.info("Delete RefApp stack")
    openstack_client_manager.stack.delete([stack_name])

    wait = wait_utils.Waiter(openstack_client_manager.os_manager, 3600)

    def wait_refapp_stack_delete():
        response = openstack_client_manager.stack.show(
            [stack_name])
        return 'Stack not found' in response

    commons.LOG.info("Wait for RefApp heat stack to delete")
    wait.wait(wait_refapp_stack_delete)

    commons.LOG.info("Refapp heat stack successfully deleted")
