import json

from si_tests.utils import utils
from si_tests.deployments.utils import file_utils, commons, helpers


@utils.log_method_time()
def generate_ssl_certificates(os_manager, timeout):
    local_domain = helpers.get_local_domain(os_manager)
    commons.LOG.info("Prepare ssl config")
    certs_path = file_utils.get_context("certs")
    csr_yaml_path = file_utils.join(certs_path, "api-csr.yaml")
    csr_config = file_utils.get_yaml_content(csr_yaml_path)

    csr_config["CN"] = "*.openstack.svc.{}".format(local_domain)

    csr_config["hosts"].append("*.openstack.svc.{}".format(local_domain))

    csr_config_path = "api-csr.json"
    with open(file_utils.join(certs_path, csr_config_path), "w") as f:
        json.dump(csr_config, f)

    cffsl_binary = utils.get_binary_path("cfssl")
    cfssljson_binary = utils.get_binary_path("cfssljson")

    commons.LOG.info("Generate ssl certificates")
    executor = commons.get_local_executor()
    result = executor.execute(
        f"{cffsl_binary} gencert -ca=ca.crt -ca-key=ca.key {csr_config_path} |"
        f"{cfssljson_binary} -bare \"api\"",
        cwd=certs_path)

    commons.LOG.info("%s", result.stderr)
    assert result.exit_code == 0, result
