from si_tests import settings
from si_tests import logger

import pytest
import re
LOG = logger.logger


@pytest.fixture(scope='session')
def autoset_migration_mode(kaas_manager):
    yield

    ns = kaas_manager.get_namespace(settings.TARGET_NAMESPACE)
    cluster = ns.get_cluster(settings.TARGET_CLUSTER)
    if not re.search(r"mos.?-.*", cluster.clusterrelease_version):
        msg = "Skipping annotation configuration for non-Openstack clusters"
        LOG.info(msg)
        return
    LOG.info("Setting migration mode for different compute types")
    dpdk_machines = [m for m in cluster.get_machines(machine_type="worker") if
                     m.get_k8s_node().data.get('metadata', {}).get('labels', {}).get(
                         'openstack-compute-node-dpdk', '') == 'enabled']
    nova_machines = [m for m in cluster.get_machines(machine_type="worker") if
                     m.get_k8s_node().data.get('metadata', {}).get('labels', {}).get(
                         'openstack-compute-node', '') == 'enabled']
    nova_machines = [m for m in nova_machines if m not in dpdk_machines]
    if dpdk_machines:
        if len(dpdk_machines) == 1:
            LOG.info("Only 1 dpdk compute found. Setting migration mode to SKIP")
            dpdk_machines[0].add_k8s_node_annotations(
                annotations={'openstack.lcm.mirantis.com/instance_migration_mode': 'skip'})
        else:
            LOG.info("More then 1 dpdk compute found. Setting migration mode to LIVE")
            for machine in dpdk_machines:
                machine.add_k8s_node_annotations(
                    annotations={'openstack.lcm.mirantis.com/instance_migration_mode': 'live'})
    else:
        LOG.info("Machines with label openstack-compute-node-dpdk are not found. Skipping")

    if nova_machines:
        if len(nova_machines) == 1:
            LOG.info("Only 1 compute found in nova zone. Setting migration mode to SKIP")
            nova_machines[0].add_k8s_node_annotations(
                annotations={'openstack.lcm.mirantis.com/instance_migration_mode': 'skip'})
        else:
            LOG.info("More then 1 compute found in nova zone. Setting migration mode to LIVE")
            for machine in nova_machines:
                machine.add_k8s_node_annotations(
                    annotations={'openstack.lcm.mirantis.com/instance_migration_mode': 'live'})
    else:
        LOG.info("Machines with label openstack-compute-node are not found")
