import exec_helpers
import yaml
import json

from packaging import version
from si_tests import settings
from si_tests import logger
import si_tests.utils.templates as utils

LOG = logger.logger


def test_iam_bdd(kaas_manager, iam_ip_port, keycloak_ip):
    pod_name = settings.IAM_BDD_TEST_POD_NAME
    # this is hardcoded for now
    pod_namespace = 'kaas'
    pod_list = kaas_manager.api.api_core.list_namespaced_pod(
        namespace='kaas',
        label_selector='app.kubernetes.io/name=iam-api').to_dict()['items'][0]
    actual_version = pod_list['spec']['containers'][0]['image'].split(':')[1]
    if version.parse(actual_version) < version.parse('0.3.11'):
        LOG.debug("Actual IAM version is {0}. 0.3.11 iam-bdd image tag "
                  "will be used instead if IAM_VERSION env variable "
                  "is empty".format(actual_version))
        actual_version = '0.3.11'
    iam_version = settings.IAM_VERSION or actual_version
    cdn_region = settings.CDN_REGION
    artifacts_region_map = settings.ARTIFACTS_REGION_MAPPING
    iam_image_base = artifacts_region_map[cdn_region]["images_base_url"]
    options = {'IAM_API_URL': iam_ip_port,
               'IAM_BDD_TEST_IMAGE': iam_image_base + "/" +
               settings.IAM_BDD_TEST_IMAGE + ":" +
               iam_version,
               'IAM_KEYCLOAK_URL': "https://" + keycloak_ip}
    templates = utils.render_template(settings.IAM_BDD_TEST_POD_YAML, options)
    LOG.debug(templates)
    json_body = json.dumps(yaml.load(templates, Loader=yaml.SafeLoader))
    pod = kaas_manager.api.pods.create(name=pod_name,
                                       namespace=pod_namespace,
                                       body=json.loads(json_body))
    pod.wait_phase('Running')
    pod.wait_test(filepath='./test_*')

    pod.cp_from_pod(source_dir=settings.IAM_BDD_TEST_REPORTS_DIR)
    exec_helpers.Subprocess().check_call(
        "test -f artifacts/junit_*.xml && "
        "mv artifacts/junit_* artifacts/iam_bdd_report.xml")

    pod.delete(async_del=True)
