#    Copyright 2020 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from si_tests import logger
from si_tests import settings

from si_tests.managers import bootstrap_manager

LOG = logger.logger


class TestProvisionBmRegionalClusters:
    def test_deploy_bm_regional(self, kaas_manager,
                                openstack_client, show_step):
        """Deploy Regional cluster on KaaS mgmt

        Scenario:
            1. Prepare seed node
            2. Prepare seed node, for BM management cluster deployment
            3. Pass Region var's, and run bootsrap.sh deploy_regional
            4. Check expected resources after REGION deploy, on MGMT cluster
            5. Check expected resources on REGION-mgmt cluster
            6. Download mgmt artifacts
            7. Download bm-mgmt(region) artifacts
        """

        bootstrap = bootstrap_manager.BootstrapManager()

        show_step(1)
        bootstrap.step_003_prepare_bm_seed_node(openstack_client)

        # This step doesn't start `bootstrap.sh all` if
        # BM_DEPLOY_REGIONAL = True
        # Only render templates for bm
        show_step(2)
        bootstrap.step_004_deploy_kaas_bm_cluster_physical(
            openstack_client, settings.ENV_CONFIG_NAME or settings.ENV_NAME)
        show_step(3)
        bootstrap.step_004_deploy_kaas_regional(openstack_client,
                                                region=settings.BM_REGION)
        show_step(4)
        bootstrap.step_006_wait_for_pods()
        bootstrap.step_006_postdeployment_checks(openstack_client)

        show_step(5)
        ns = kaas_manager.get_namespace(settings.REGION_NAMESPACE)
        regional_cluster = ns.get_cluster(settings.REGIONAL_CLUSTER_NAME)
        regional_cluster.check.check_helmbundles()
        regional_cluster.check.check_persistent_volumes_mounts()
        regional_cluster.check.check_machines_status()
        regional_cluster.check.check_cluster_nodes(timeout=900)
        regional_cluster.check.check_actual_expected_kernel_versions()
        regional_cluster.store_k8s_artifacts()

        # Check/wait for correct docker service replicas in cluster
        regional_cluster.check.check_actual_expected_docker_services()
        regional_cluster.check.check_k8s_pods()
        regional_cluster.check.check_cluster_readiness()
        regional_cluster.check.check_actual_expected_pods(timeout=1200,
                                                          check_all_nss=True)
        show_step(6)
        bootstrap.step_007_download_bootstrap_artifacts(openstack_client)
        show_step(7)
        bootstrap.step_007_download_bootstrap_artifacts(openstack_client,
                                                        nsname=settings.REGION_NAMESPACE, # noqa
                                                        clname=settings.REGIONAL_CLUSTER_NAME)  # noqa

    def test_deploy_bm_regional_virtual_phase2(self, kaas_manager,
                                               openstack_client,
                                               show_step):
        """Bootstrap KaaS_BM on OpenStack VM

        Scenario:
            4. Prepare for kaas-bm install, using ansible playbooks
            5. Rsync si-tests fetched kubeconfigs
            6. Deploy kaas-bm region
            7. Check expected resources after REGION deploy, on MGMT cluster
            8. Check expected resources on REGION-mgmt cluster
            9. Download artifacts resources- from mgmt
            10. Download artifacts resources- from region
        """

        bootstrap = bootstrap_manager.BootstrapManager()
        ###
        # This step doesn't start `bootstrap.sh all` if
        # BM_DEPLOY_REGIONAL = True
        # Only render templates for bm
        show_step(4)
        bootstrap.step_004_deploy_kaas_bm_mgmt_cluster(openstack_client)
        show_step(5)
        bootstrap.sync_configs_to_seed_PRODX_15087(openstack_client)
        show_step(6)
        bootstrap.step_004_deploy_kaas_regional(openstack_client,
                                                region=settings.BM_REGION)
        show_step(7)
        bootstrap.step_006_wait_for_pods()
        bootstrap.step_006_postdeployment_checks(openstack_client)
        show_step(8)
        ns = kaas_manager.get_namespace(settings.REGION_NAMESPACE)
        regional_cluster = ns.get_cluster(settings.REGIONAL_CLUSTER_NAME)
        regional_cluster.check.check_helmbundles()
        regional_cluster.check.check_persistent_volumes_mounts()
        regional_cluster.check.check_machines_status()
        regional_cluster.check.check_cluster_nodes(timeout=900)
        regional_cluster.store_k8s_artifacts()

        # Check/wait for correct docker service replicas in cluster
        regional_cluster.check.check_actual_expected_docker_services()
        regional_cluster.check.check_k8s_pods()
        regional_cluster.check.check_actual_expected_pods(timeout=1200)
        regional_cluster.check.check_cluster_readiness()
        show_step(9)
        bootstrap.step_007_download_bootstrap_artifacts(openstack_client)
        show_step(10)
        bootstrap.step_007_download_bootstrap_artifacts(openstack_client,
                                                        nsname=settings.REGION_NAMESPACE, # noqa
                                                        clname=settings.REGIONAL_CLUSTER_NAME)  # noqa
