from si_tests import settings
from si_tests import logger
from si_tests.utils import exceptions
from si_tests.utils import waiters

LOG = logger.logger


def test_detach_byo_ucp(kaas_manager, iam_api):
    # detaching cluster
    cluster_name = settings.TARGET_CLUSTER
    ns = kaas_manager.get_namespace(settings.TARGET_NAMESPACE)
    ucp_cluster = ns.get_cluster(cluster_name)
    mgmt_k8s = kaas_manager.api
    parent_k8s = ucp_cluster.get_parent_cluster().k8sclient
    ucp_k8sclient = ucp_cluster.k8sclient

    # use our usual kaas user
    writer_password = kaas_manager.get_keycloak_user_password('writer')
    client = iam_api.connect(username='writer',
                             password=writer_password)
    scopes = [x for x in client.list_scopes() if cluster_name in x.name]
    LOG.info("IAM scopes before detach: {}".format(scopes))

    ucp_cluster.delete()

    LOG.info("Checking that cluster object is deleted")
    waiters.wait(
        lambda: len(mgmt_k8s.kaas_clusters.list(
            namespace=ns.name)) == 0, timeout=1800, interval=10)

    def list_cluster_scope():
        try:
            client = iam_api.connect(username='writer', password=writer_password)
            scopes_list = [x for x in client.list_scopes() if cluster_name in x.name]
        except Exception as e:
            LOG.info(e)
            client = iam_api.connect(username='writer', password=writer_password)
            scopes_list = [x for x in client.list_scopes() if cluster_name in x.name]
        return scopes_list
    try:
        waiters.wait(
            lambda: len(list_cluster_scope()) == 0,
            timeout=500, interval=20)
    except exceptions.TimeoutError:
        client = iam_api.connect(username='writer',
                                 password=writer_password)
        scopes = [x for x in client.list_scopes() if cluster_name in x.name]
        LOG.info("IAM scopes after detach: {}".format(scopes))
        assert len(scopes) == 0,\
            "Not all scopes have been deleted: {}".format(scopes)

    assert len(mgmt_k8s.kaas_machines.list(namespace=ns.name)) == 0, \
        "Not all machines were deleted " \
        "{}".format(mgmt_k8s.kaas_machines.list(namespace=ns.name))

    assert len(parent_k8s.kaas_helmbundles.list(namespace=ns.name)) == 0, \
        "Not all helmbundles were deleted " \
        "{}".format(parent_k8s.kaas_helmbundles.list(namespace=ns.name))

    assert len(parent_k8s.kaas_lcmclusters.list(namespace=ns.name)) == 0, \
        "Not all lcmclusters were deleted " \
        "{}".format(parent_k8s.kaas_lcmclusters.list(namespace=ns.name))

    assert len(parent_k8s.kaas_lcmmachines.list(namespace=ns.name)) == 0, \
        "Not all lcmmachines were deleted " \
        "{}".format(parent_k8s.kaas_lcmmachines.list(namespace=ns.name))

    LOG.info("Make sure metrics-server pod is removed from standalone UCP")
    waiters.wait(
        lambda: len(ucp_k8sclient.pods.list(
            namespace='kube-system',
            name_prefix='metrics-server')) == 0,
        timeout=140, interval=10)

    LOG.info("Make sure helm-controller pods are removed from standalone UCP")
    waiters.wait(
        lambda: len(ucp_k8sclient.pods.list(
            namespace='kube-system',
            name_prefix='helm-controller')) == 0,
        timeout=140, interval=10)

    ns.delete()
    ns.wait_for_deletion()

    LOG.info("Make sure namespace is deleted")
    waiters.wait(
        lambda: not mgmt_k8s.namespaces.present(name=ns.name),
        timeout=100, interval=10)

    LOG.info("Checking that byocredentials object is deleted")
    waiters.wait(
        lambda: len(mgmt_k8s.kaas_byocredentials.list(
            namespace=ns.name)) == 0, timeout=100, interval=10)
